<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/28/16
 * Time: 8:05 AM
 */

namespace qfq;

require_once(__DIR__ . '/Sanitize.php');

class Support {

    /**
     * @param array $queryArray Empty or prefilled assoc array with url parameter
     */
    public static function appendTypo3ParameterToArray(array &$queryArray) {
//        if (isset($_GET['id']))
//            $queryArray['id'] = self::getCurrentPage();
        if (isset($GLOBALS["TSFE"]->id))
            $queryArray['id'] = $GLOBALS["TSFE"]->id;

        if (isset($_GET['L']))
            $queryArray['L'] = $_GET['L'];

        if (isset($_GET['type']))
            $queryArray['type'] = $_GET['type'];

    }

    /**
     * Builds a urlencoded query string of an assoc array.
     *
     * @param array $queryArray
     * @return string Querystring (e.g.: id=23&type=99
     */
    public static function arrayToQueryString(array $queryArray) {
        $items = array();

        foreach ($queryArray as $key => $value) {
            if (is_int($key)) {
                $items[] = urlencode($value);
            } else {
                $items[] = $key . '=' . urlencode($value);
            }
        }

        return implode('&', $items);
    }

    /**
     * Extract Tag from $tag (might contain further attributes) and wrap it around $value. If $flagOmitEmpty==true && $value=='': return ''.
     *
     * @param $tag
     * @param $value
     * @param bool|false $omitIfValueEmpty
     * @return string
     */
    public static function wrapTag($tag, $value, $omitIfValueEmpty = false) {
        if ($omitIfValueEmpty && $value === "")
            return '';

        // a) <div class="container-fluid">, b) <label>
        $arr = explode(' ', $tag);

        $tagPlain = (count($arr) === 1) ? substr($arr[0], 1, strlen($arr[0]) - 2) : substr($arr[0], 1);
        $closing = '</' . $tagPlain . '>';

        return $tag . $value . $closing;
    }

    /**
     * Wraps some $inner fragment with a CSS styled $tooltipText . CSS is configured in 'Resources/Public/qfq-jqw.css'.
     *
     * Based on: http://www.w3schools.com/howto/howto_css_tooltip.asp
     *
     * @param $inner
     * @param $tooltipText
     * @return string
     */
    public static function appendTooltip($inner, $tooltipText) {

        return $inner . "<img src='" . GFX_INFO . "' title='$tooltipText'>";
    }

    /**
     * Search for the parameter $needle in $haystack. The argumenst has to be seperated by ','.
     *
     * Returns the false if not found or index of found place. Be carefull: use unary operator to compare for 'false'
     *
     * @param $needle
     * @param $haystack
     * @return boolean     true if found, else false
     */
    public static function findInSet($needle, $haystack) {
        $arr = explode(',', $haystack);

        return array_search($needle, $arr) !== false;

    }

    /**
     * Converts a dateTime String to the english format:
     * 1.2.79 > 1979-02-01 00:00:00
     * 01.02.13 3:24 >  1979-02-01 03:24:00
     * 1.2.1979 14:21:5 > 1979-02-01 14:21:05
     *
     * @param $dateTimeString
     * @return string
     * @throws UserException
     */
    public static function dateTime2mysql($dateTimeString) {
        $dateRaw = '';
        $timeRaw = '';

        $tmpArr = explode(' ', $dateTimeString);
        switch (count($tmpArr)) {
            case 0:
                return '';

            case 1:
                if (strpos($tmpArr[0], '.') === false) {
                    $timeRaw = $tmpArr[0];
                } else {
                    $dateRaw = $tmpArr[0];
                }
                break;

            case 2:
                $dateRaw = $tmpArr[0];
                $timeRaw = $tmpArr[1];
                break;

            default:
                throw new UserException('Date/time format not recognised.', ERROR_DATE_TIME_FORMAT_NOT_RECOGNISED);
                break;
        }

        if ($dateRaw === '') {
            $date = '0000-00-00';
        } else {
            $tmpArr = explode('.', $dateRaw);
            if ($tmpArr[2] < 70) {
                $tmpArr[2] = 2000 + $tmpArr[2];
            } elseif ($tmpArr[2] < 100) {
                $tmpArr[2] = 1900 + $tmpArr[2];
            }
            $date = sprintf("%04d-%02d-%02d", $tmpArr[2], $tmpArr[1], $tmpArr[0]);
        }

        if ($timeRaw === '') {
            $time = '00:00:00';
        } else {
            $tmpArr = explode(':', $timeRaw);
            switch (count($tmpArr)) {
                case 2:
                    $time = sprintf("%02d:%02d:00", $tmpArr[0], $tmpArr[1]);
                    break;
                case 3:
                    $time = sprintf("%02d:%02d:%02d", $tmpArr[0], $tmpArr[1], $tmpArr[2]);
                    break;
                default:
                    throw new UserException('Date/time format not recognised.', ERROR_DATE_TIME_FORMAT_NOT_RECOGNISED);
                    break;
            }
        }

        return $date . ' ' . $time;
    }
}