<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/28/16
 * Time: 9:58 AM
 */

namespace qfq;

require_once(__DIR__ . '/../../qfq/helper/Support.php');
require_once(__DIR__ . '/../../qfq/BuildFormPlain.php');
require_once(__DIR__ . '/../../qfq/Evaluate.php');

class FakeTSFE {
    public $id = 1;
}

class SupportTest extends \PHPUnit_Framework_TestCase {

    public function testAppendTypo3ParameterToArray() {

        unset($_GET);

        $GLOBALS["TSFE"] = new FakeTSFE();
        $queryArray = array();
        Support::appendTypo3ParameterToArray($queryArray);
        $this->assertEquals(['id' => 1], $queryArray);

        $queryArray = ['id' => '1', 'a' => '100'];
        $expected = $queryArray;
        Support::appendTypo3ParameterToArray($queryArray);
        $this->assertEquals($expected, $queryArray);

        $_GET['L'] = 1;
        $_GET['type'] = 100;
        $queryArray = ['id' => '1', 'a' => '100'];
        $expected = array_merge($_GET, $queryArray);
        Support::appendTypo3ParameterToArray($queryArray);
        $this->assertEquals($expected, $queryArray);

        unset($_GET);
        $_GET['L'] = 1;
        $queryArray = ['id' => '1', 'a' => '100'];
        $expected = array_merge($_GET, $queryArray);
        Support::appendTypo3ParameterToArray($queryArray);
        $this->assertEquals($expected, $queryArray);

        unset($_GET);
        $_GET['type'] = 1;
        $queryArray = ['id' => '1', 'a' => '100'];
        $expected = array_merge($_GET, $queryArray);
        Support::appendTypo3ParameterToArray($queryArray);
        $this->assertEquals($expected, $queryArray);

    }

    public function testArrayToQueryString() {
        $actual = Support::arrayToQueryString(array());
        $expected = '';
        $this->assertEquals($expected, $actual);

        $actual = Support::arrayToQueryString(['id', 'name']);
        $expected = '';
        $this->assertEquals('id&name', $actual);

        $actual = Support::arrayToQueryString(['id' => '55', 'name']);
        $expected = '';
        $this->assertEquals('id=55&name', $actual);

        $actual = Support::arrayToQueryString(['id' => '55', 'name' => 'Heinz']);
        $expected = '';
        $this->assertEquals('id=55&name=Heinz', $actual);

        $actual = Support::arrayToQueryString(['id' => '55', 'name' => 'He<b>inz']);
        $expected = '';
        $this->assertEquals('id=55&name=He%3Cb%3Einz', $actual);

    }

    public function testWrapTag() {
        $build = new \qfq\BuildFormPlain(array(), array(), array());

        $result = Support::wrapTag("<p class='test'>", 'Hello World', false);
        $this->assertEquals("<p class='test'>Hello World</p>", $result);

        $result = Support::wrapTag("<p class='test'>", 'Hello World', true);
        $this->assertEquals("<p class='test'>Hello World</p>", $result);

        $result = Support::wrapTag("<p class='test'>", '', false);
        $this->assertEquals("<p class='test'></p>", $result);

        $result = Support::wrapTag("<p class='test'>", '', true);
        $this->assertEquals('', $result);

        $result = Support::wrapTag("<p>", '', true);
        $this->assertEquals('', $result);

        $result = Support::wrapTag("<p>", '', false);
        $this->assertEquals('<p></p>', $result);

        $result = Support::wrapTag("<p>", 'Hello World', true);
        $this->assertEquals('<p>Hello World</p>', $result);

        $result = Support::wrapTag("<p>", 'Hello World', false);
        $this->assertEquals('<p>Hello World</p>', $result);

    }

    public function testDateTime2mysql() {

        // date
        $this->assertEquals('2069-12-31 00:00:00', Support::dateTime2mysql('31.12.69'));
        $this->assertEquals('1970-01-01 00:00:00', Support::dateTime2mysql('1.1.70'));
        $this->assertEquals('1999-02-01 00:00:00', Support::dateTime2mysql('01.02.99'));
        $this->assertEquals('2079-02-01 00:00:00', Support::dateTime2mysql('1.2.2079'));
        $this->assertEquals('2079-02-01 00:00:00', Support::dateTime2mysql('01.02.2079'));

        // time
        $this->assertEquals('0000-00-00 03:04:00', Support::dateTime2mysql('3:4'));
        $this->assertEquals('0000-00-00 03:04:00', Support::dateTime2mysql('03:04'));
        $this->assertEquals('0000-00-00 03:14:00', Support::dateTime2mysql('3:14'));
        $this->assertEquals('0000-00-00 13:14:00', Support::dateTime2mysql('13:14'));
        $this->assertEquals('0000-00-00 13:14:01', Support::dateTime2mysql('13:14:1'));
        $this->assertEquals('0000-00-00 13:14:41', Support::dateTime2mysql('13:14:41'));

        // date time
        $this->assertEquals('2069-02-01 01:02:00', Support::dateTime2mysql('1.2.69 1:2'));
        $this->assertEquals('2016-12-31 23:48:59', Support::dateTime2mysql('31.12.2016 23:48:59'));

    }
}
